#ifndef BSP_H
#define BSP_H

////////////////////////////////////////////////////////////////

#include <vector>
#include <map>

#include "common.hpp"

////////////////////////////////////////////////////////////////

namespace CoD
{
    ////////////////////////////////////////////////////////////////

    union DoubleWord
    {
        unsigned char b[4];
        int i;
        unsigned int u, x;
        float f, g;

        DoubleWord()
        {
            b[0] = 0;
            b[1] = 0;
            b[2] = 0;
            b[3] = 0;
        }
    };

    ////////////////////////////////////////////////////////////////

    struct Index;
    struct Plane;
    struct Texture;
    struct TextureGroup;
    struct Vertex;
    struct Triangle;
    struct Entity;

    ////////////////////////////////////////////////////////////////

    class BSP;

    typedef void (BSP::*LUMP)();

    class BSP
    {
    public:
        BSP(char *self, char *filename);
        ~BSP();

        void cod1();
        void cod2();

        void read();

        void lump_textures();
        void lump_planes();
        void lump_texture_groups();
        void lump_vertexes();
        void lump_triangles();
        void lump_entities();

    private:
        bool verbose;

        Data data;
        char *i, *size;
        std::map<unsigned int, LUMP> lump;

        std::vector<Index> index;

    public:
        std::string path;
        std::string map;

        int cod_version;
        unsigned int cod_size;

        std::vector<Plane> planes;
        std::vector<TextureGroup> texture_groups;
        std::vector<Vertex> vertexes;
        std::vector<Triangle> triangles;
        std::vector<Entity> entities;
    };

    ////////////////////////////////////////////////////////////////

    struct Index
    {
        unsigned int offset;
        unsigned int size;

        Index(unsigned int offset = 0, unsigned int size = 0) :
            offset(offset),
            size(size)
        {}
    };

    struct Texture
    {
        char name[64];
        DoubleWord flag[2];

        Texture()
        {
            memset(name, 0x00, sizeof(name[0]) * 64);
        }
    };

    struct Plane
    {
        int normal[3];
        int distance;

        Plane() :
            distance(0)
        {
            normal[0] = 0;
            normal[1] = 0;
            normal[2] = 0;
        }
    };

    struct TextureGroup
    {
        unsigned int texture;

        unsigned int vertex_offset;
        unsigned int vertex_length;

        unsigned int triangle_offset;
        unsigned int triangle_length;

        TextureGroup() :
            texture(0),
            vertex_offset(0),
            vertex_length(0),
            triangle_offset(0),
            triangle_length(0)
        {}
    };

    struct Vertex
    {
        float position[3];
        unsigned char alpha;

        Vertex() :
            alpha(255)
        {
            position[0] = 0.0f;
            position[1] = 0.0f;
            position[2] = 0.0f;
        }
    };

    struct Triangle
    {
        unsigned int vertex[3];

        Triangle()
        {
            vertex[0] = 0;
            vertex[1] = 0;
            vertex[2] = 0;
        }
    };

    struct Entity
    {
        char *data;

        Entity()
        {}
    };

    ////////////////////////////////////////////////////////////////
}

////////////////////////////////////////////////////////////////

#endif // BSP_H
