#include <iostream>

#include "window.hpp"
#include "draw.hpp"

#define M_PI 3.1415926535897932384626433832795

////////////////////////////////////////////////////////////////

#define VERBOSE 0

namespace CoD
{
    ////////////////////////////////////////////////////////////////

    Draw::Draw(char *_working_directory) :
        bsp(NULL),
        working_directory(_working_directory),
        fullscreen(false),
        loading_alpha(1.0),
        show_loading(true),
        screenshot_i(1),
        information_alpha(0.0),
        information_i(0)
    {
	resolution.width = 800;
	resolution.height = 600;
	fullscreen = 0;

        x = 0.0;
        y = 0.0;
        z = 0.0;
        xp = 0.0;
        yp = 0.0;
        zp = 0.0;
        xpp = 0;
        ypp = 0;
        zpp = 0;
        x_angle = 0.0;
        z_angle = 0.0;
        accelerate = false;
        acceleration = 1.0;
        active = true;
        focussed = true;

        for (unsigned int n = 0; n < 256; n++)
            keys[n] = false;

        reset();
        initialise();
    }

    Draw::~Draw()
    {
    }

    void Draw::run()
    {
        if (!bsp)
        {
            return;
        }

        while (active)
        {
            input();
            process();
            Window::pre_frame();

            draw();

            Window::post_frame();
            //Window::delay();
        }

        Window::shutdown();
    }

    void Draw::draw()
    {
        to_3d();

        glLoadIdentity();
        gluLookAt(x, y, z, x + (sin(x_angle) * cos(z_angle)), y + (cos(x_angle) * cos(z_angle)), z + sin(z_angle), 0.0f, 0.0f, 1.0f);

        glDisable(GL_TEXTURE_2D);

        if (use_blending != 1 && use_blending != 2)
        {
            glEnable(GL_DEPTH_TEST);
            glDisable(GL_BLEND);
        }
        else if (use_blending != 2)
        {
            glEnable(GL_BLEND);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        }
        else
        {
            glDisable(GL_DEPTH_TEST);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE);
        }

        if (show_planes || show_single_plane)
        {
            draw_planes();
        }

        if (show_triangles)
        {
            draw_triangles();
        }

        if (show_lines)
        {
            draw_lines();
        }

        if (show_vertexes)
        {
            draw_vertexes();
        }

        to_2d();

        glLoadIdentity();

        glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
        glEnable(GL_TEXTURE_2D);
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glDisable(GL_DEPTH_TEST);

/*
<< "CoD BSP Renderer; Information and statistics\n\n"
<< "Mapname: " << bsp->map << '\n'
<< "CoD version: " << bsp->cod_version << "\n\n"
<< "FPS: " << ((Window::stable_fps() * 10.0) / 10.0) << '\n'
<< "SPF: " << ((Window::stable_spf() * 100000.0) / 100.0) << " ms\n\n"
<< "Vertexes: " << bsp->vertexes.size() << '\n'
<< "Triangles: " << bsp->triangles.size() << '\n'
<< "Texture groups: " << bsp->texture_groups.size() << '\n'
<< "Planes: " << bsp->planes.size() << '\n';

<< "\nCurrent texture group:\n"
<< "Texture group ID: " << texture_group_i << '\n'
<< "Texture ID: " << bsp->texture_groups[texture_group_i].texture << '\n'
<< "Vertexes: " << bsp->texture_groups[texture_group_i].vertex_length << '\n'
<< "Triangles: " << bsp->texture_groups[texture_group_i].triangle_length << '\n';

<< "CoD BSP Renderer; Key combinations\n\n"

<< "F1: show key combinations\n"
<< "F2: show credits and license\n"
<< "F12: make a screenshot\n\n"

<< "i: show information and statistics\n"
<< "r: reset settings to default\n\n"

<< "w: move forward\n"
<< "a: move left\n"
<< "s: move backward\n"
<< "d: move right\n"
<< "q: move down\n"
<< "e: move up\n\n"

<< "v: show / hide vertexes\n"
<< "l: show / hide lines\n"
<< "t: show / hide triangles\n"
<< "g: enable / disable single texture group display mode\n"
<< "p: cycle plane display mode (not yet working)\n"
<< "b: cycle blending mode\n\n"

<< "page up: increase near plane, increase far plane\n"
<< "page down: decrease near plane, decrease far plane\n\n"

<< "[: increase vertex size\n"
<< "]: decrease vertex size\n\n"

<< "semicolon: increase line width\n"
<< "quote: decrease line width\n\n"

<< "comma: increase opacity\n"
<< "period: decrease opacity\n\n"

<< "right: increase texture group ID\n"
<< "left: decrease texture group ID\n\n"

<< "up: increase plane ID (not yet working)\n"
<< "down: decrease plane ID (not yet working)\n\n"

<< "shift: multiplies move and increase / decrease speed by 4\n\n"

<< "mouse click: gain / lose focus\n";


<< "CoD BSP Renderer; Credits and license\n\n"
<< "Together with CoDEmanX I have developed this tool.\n"
<< "Also credits go to Regolith for his work on the bsp file format in the past.\n\n"
<< "Copyright (C) 2008 Taco de Wolff\n\n"
<< "Taco aka Daevius (http://forum.daevius.com)\n";
*/
    }

    ////////////////////////////////////////////////////////////////

    #include "draw_initialise.cpp"
    #include "draw_draw.cpp"
    #include "draw_control.cpp"

    ////////////////////////////////////////////////////////////////

    void Draw::screenshot()
    {
	/*
        screenshot_i++;
        unsigned int size = resolution.width * resolution.height * 3;
        unsigned char *pixels = new unsigned char[size];
        glPixelStorei(GL_PACK_ALIGNMENT, 1);
        glReadPixels(0, 0, resolution.width, resolution.height, GL_RGB, GL_UNSIGNED_BYTE, pixels);
        corona::Image *image = corona::CreateImage(resolution.width, resolution.height, corona::PF_R8G8B8, pixels);
        image = corona::FlipImage(image, corona::CA_X);
        corona::SaveImage(filename.c_str(), corona::FF_PNG, image);
        delete image;
	*/
    }

    double Draw::multiplier()
    {
        return (Window::stable_spf() * frame_reference);
    }

    void parse_settings(std::string filename, Resolution &resolution, bool &fullscreen)
    {
        FILE *settings;
        settings = fopen(filename.c_str(), "rb+");
        if (!settings)
        {
            std::cerr << "ERROR: Could not open settings file '" << filename << "'.\n";
            return;
        }

        // reading settings file
        fseek(settings, 0, SEEK_END);
        Data data = Data(new char[ftell(settings)], ftell(settings));
        fseek(settings, 0, SEEK_SET);

        if (fread(data.buffer, 1, data.size, settings) != data.size)
        {
            std::cerr << "ERROR: Could not read settings file '" << filename << "'.\n";
            delete[] data.buffer;
            fclose(settings);
            return;
        }
        fclose(settings);

        resolution = Resolution(1024, 768);
        fullscreen = false;

        char *i = data.buffer, *j = data.buffer, *k = data.buffer;
        while (1) // file
        {
            if (*i == '\r' || *i == '\n' || !(((size_t) i - (size_t) data.buffer) < data.size))
            {
                std::string previous_column;
                while (1) // line
                {
                    if (*j == '=' || !(j < i))
                    {
                        std::string column;
                        while (1) // column
                        {
                            if (!(k < j))
                            {
                                break;
                            }

                            if (*k != ' ')
                            {
                                column.push_back(*k);
                            }
                            k++;
                        } //column

                        if (previous_column.length() && column.length())
                        {
                            if (previous_column == "width")
                            {
                                resolution.width = to_int(column);
                            }
                            else if (previous_column == "height")
                            {
                                resolution.height = to_int(column);
                            }
                            else if (previous_column == "fullscreen")
                            {
                                if (to_int(column))
                                {
                                    fullscreen = true;
                                }
                            }
                        }
                        previous_column = column;

                        if ((j < i) && *j == '=')
                        {
                            j++;
                        }

                        if (!(j < i))
                        {
                            break;
                        }

                        k = j;
                    }
                    j++;
                } // line

                if ((((size_t) i - (size_t) data.buffer) < data.size) && *i == '\r')
                {
                    i++;
                }

                if ((((size_t) i - (size_t) data.buffer) < data.size) && *i == '\n')
                {
                    i++;
                }

                if (!(((size_t) i - (size_t) data.buffer) < data.size))
                {
                    break;
                }

                j = i;
                k = j;
            }
            i++;
        } // file

        delete[] data.buffer;
    }
}
