#ifndef DRAW_H
#define DRAW_H

////////////////////////////////////////////////////////////////

#include <limits>
#include <fstream>
#include <map>

//#include <boost/filesystem.hpp>
#include <sdl/SDL.h>
#include <sdl/SDL_opengl.h>
//#include <corona/corona.h>

#include "common.hpp"
#include "bsp.hpp"

////////////////////////////////////////////////////////////////

namespace CoD
{
    #define N_COLOURS 12
    const double colour[N_COLOURS][3] = {
        {1.0, 0.0, 0.0},
        {0.0, 1.0, 0.0},
        {0.0, 0.0, 1.0},
        {1.0, 1.0, 0.0},
        {0.0, 1.0, 1.0},
        {1.0, 0.0, 1.0},
        {1.0, 0.5, 0.0},
        {0.5, 1.0, 0.0},
        {0.0, 1.0, 0.5},
        {0.0, 0.5, 1.0},
        {0.5, 0.0, 1.0},
        {1.0, 0.0, 0.5}
    };

    const unsigned char text_background[] = {
        0x00, 0x00, 0x00, 0xc0,
        0x00, 0x00, 0x00, 0x80,
        0x00, 0x00, 0x00, 0x80,
        0x00, 0x00, 0x00, 0xc0,
    };

    const double epsilon = 0.001;
    const double move_speed = 2.67;
    const double rotation_speed = 1.0;
    const double shift = 4.0;
    const double friction = 0.75;
    const double fade_in = 0.1;
    const double fade_out = 0.02;
    const double frame_reference = 75.0;
    const double text_border = 4.0;

    ////////////////////////////////////////////////////////////////

    struct Resolution
    {
        int width, height;

        Resolution(int width = 0, int height = 0) :
            width(width),
            height(height)
        {
        }
    };

    ////////////////////////////////////////////////////////////////

    void parse_settings(std::string filename, Resolution &resolution, bool &fullscreen);

    ////////////////////////////////////////////////////////////////

    class Draw
    {
    public:
        Draw(char *_working_directory);
        ~Draw();

        ////////

    private:
        void initialise();
        void loading();

    public:
        void run();

        ////////

        void set_bsp(BSP &ibsp);

    private:
        void to_2d();
        void to_3d();

        void reset();

        ////////

        void input();
        void process();
        void draw();

        ////////

        void draw_planes();
        void draw_triangles();
        void draw_lines();
        void draw_vertexes();
        void draw_text(std::string text, float x, float y, float alpha);

        ////////

        void screenshot();
        double multiplier();

    private:
        BSP *bsp;

        std::string working_directory;
        std::map<std::string, GLuint> textures;

        ////////

        double x, y, z;
        double xp, yp, zp;
        int xpp, ypp, zpp;
        double x_angle, z_angle;

        double clip_near, clip_far, point_size, line_width, alpha;

        bool accelerate;
        double acceleration;

        ////////

        bool keys[256];

        bool active, focussed, fullscreen;
        unsigned int use_blending;
        double loading_alpha, information_alpha;
        bool show_information, show_vertexes, show_triangles, show_lines, show_planes, show_loading;
        bool show_single_texture_group, show_single_plane;
        unsigned int texture_group_i, plane_i, screenshot_i, information_i;

        Resolution resolution;

        ////////
    };
}

////////////////////////////////////////////////////////////////

#endif // DRAW_H
