#ifndef FONT_H
#define FONT_H

#include <iostream>

#include <sdl/SDL_opengl.h>

//#include <freetype2/ft2build.h>
//#include FT_FREETYPE_H

enum Sharpness
{
    SHARP,
    SMOOTH
};

inline int next_po2(int value)
{
    value -= 1;

    value |= value >> 16;
    value |= value >> 8;
    value |= value >> 4;
    value |= value >> 2;
    value |= value >> 1;

    return value + 1;

	//int power_of_two = 1;
	//while (power_of_two < value)
	//{
	//    power_of_two <<= 1;
	//}
	//return power_of_two;
}

struct Character
{
    bool created;

    GLuint texture;
    GLuint list;

    float advance;
    float width, height;
    float offset_x, offset_y;

    Character()
    {
        created = false;
        texture = 0;
        list = 0;
        advance = 0.0f;
        width = height = 0.0f;
        offset_x = offset_y = 0.0f;
    }

    Character(GLuint texture, GLuint list, float advance, float width, float height, float offset_x, float offset_y) :
        created(true),
        texture(texture),
        list(list),
        advance(advance),
        width(width),
        height(height),
        offset_x(offset_x),
        offset_y(offset_y)
    {
    }
};

class Font
{
public:
    Font();
    ~Font();

    void open(char *file);
    void open(std::string file);
    void open(unsigned char *buffer, long size);

    void point_size(unsigned int size);
    void pixel_size(unsigned int size);
    void sharpness(Sharpness isharpness);

    float width(char character);
    float width(char *text);
    float width(std::string text);
    float height();

    void cache(char character);
    void cache(char *text);
    void cache(std::string text);

    void clear();

    void draw(char character);
    void draw(char *text);
    void draw(std::string text);

private:
//    FT_Library library;
 //   FT_Face face;

    Character characters[128];

public:
    float ascender;
    float descender;

private:
    float size;

    bool opened, sized, sharp;
};

#endif // FONT_H
